/*
   CCLOCK
   Copyright (C) 2003, Dominik Hackl

   ALL PARTS OF CCLOCK ARE UNDER THE TERMS OF GPL. SEE DOCUMENTATION FOR MORE INFOS.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   The author of this program may be contacted at dominik@hackl.dhs.org.
 */

#include "cclock.h"
#include "digit_frames.h"

void screen_init()
{
	if(no_color == FALSE)
	{
		printf("\e[2J\e[2;29H\e[1;37m-= CONSOLE CLOCK %s =-\e[0;0m\n", VERSION);
	}
	else
	{
		printf("\e[2J\e[2;29H-= CONSOLE CLOCK %s =-\n", VERSION);    
	}
}

void check_screen()
{
	char *tty;
	int fd, result;
	struct winsize win;
	if(!(tty = (char *)ttyname(0)) || (fd = open(tty, O_RDWR)) == -1 || ioctl(fd, TIOCGWINSZ, &win) == -1)
	{
		return;
	}
	if(win.ws_col != cols || win.ws_row != rows)
	{
		cols = win.ws_col;
		rows = win.ws_row;
	}
}

void print_digit_frame(const char *frame, unsigned int *x, unsigned int *y)
{
	unsigned int i, a;
	*x *= 10;
	for(i = 0; i <= 7 * 10; i += 10)
	{
		printf("\e[%d;%dH", (i / 10) + *y, *x);
		for(a = i; a < 10 + i; a++)
		{
			if(no_color == FALSE)
			{
				if(*(frame + a) == '#' && *(frame + a - 1) != '#')
				{
					printf("\e[1;42m");
				}
				if(*(frame + a) == '*' && *(frame + a - 1) != '*')
				{
					printf("\e[1;41m");
				}
				else if(*(frame + a) == ' ' && *(frame + a - 1) != ' ')
				{
					printf("\e[0;0m");    
				}
				printf(" ");
			}
			else
			{
				printf("%c", *(frame + a));
			}
		}
		printf("\n");
	}
}

void print_digit(const char digit, unsigned int x, unsigned int y)
{
	switch(digit)
	{
		case ':':
			print_digit_frame((const char *)colson, &x, &y);
			break;
		case '.':
			print_digit_frame((const char *)dot, &x, &y);
			break;
		case '1':
			print_digit_frame((const char *)one, &x, &y);
			break;
		case '2':
			print_digit_frame((const char *)two, &x, &y);
			break;
		case '3':
			print_digit_frame((const char *)three, &x, &y);
			break;
		case '4':
			print_digit_frame((const char *)four, &x, &y);
			break;
		case '5':
			print_digit_frame((const char *)five, &x, &y);
			break;
		case '6':
			print_digit_frame((const char *)six, &x, &y);
			break;
		case '7':
			print_digit_frame((const char *)seven, &x, &y);
			break;
		case '8':
			print_digit_frame((const char *)eight, &x, &y);
			break;
		case '9':
			print_digit_frame((const char *)nine, &x, &y);
			break;
		case '0':
			print_digit_frame((const char *)zero, &x, &y);
			break;
	}
}

void print_time(const char *curr_time, char *last_time)
{
	int n;
	for(n = 0; n < strlen(curr_time); n++)
	{
		if(last_time[n] != curr_time[n])
		{
			print_digit(curr_time[n], n, 5);
		}
	}
	strncpy(last_time, curr_time, 10);	
}

void print_date(const char *curr_date, char *last_date)
{
	int n;
	for(n = 0; n < strlen(curr_date); n++)
	{
		if(last_date[n] != curr_date[n])
		{
			print_digit(curr_date[n], n, 15);
		}
	}
	strncpy(last_date, curr_date, 10);
}

unsigned int start_clock()
{
	char curr_time[10], curr_date[10], last_time[10], last_date[10];
	time_t curr_time_tmp;
	struct tm *tm_clock;
	unsigned int n = 0, i;
	while(shutdown == FALSE)
	{
		if(n >= 10 || refresh_screen == TRUE)
		{
			check_screen();
			screen_init();
			refresh_screen = FALSE;
			bzero(last_time, 10);
			bzero(last_date, 10);
			n = 0;
		}
		if(cols < 80 || rows < 24)
		{
			printf("\e[7;20H\e[1;37mSorry, your screen is smaller than 80x24.\n");
		}
		else
		{
			time(&curr_time_tmp);
			if(gmt == TRUE)
			{
				tm_clock = gmtime(&curr_time_tmp);
			}
			else
			{
				tm_clock = localtime(&curr_time_tmp);
			}
			strftime(curr_date, 10, "%d.%m.%y", tm_clock);
			strftime(curr_time, 10, "%I:%M:%S", tm_clock);
			print_time(curr_time, last_time);
			print_date(curr_date, last_date);
		}
		usleep(update_time * 10000);
		n++;
	}
	return 0;
}

void handle_screen_change(int code)
{
	refresh_screen = TRUE;
}

void handle_quit_signal(int code)
{
	printf("\e[2J\e[0;0Hcclock got a ");
	if(code == SIGTERM)
	{
		printf("SIGTERM");
	}
	else if(code == SIGINT)
	{
		printf("SIGINT");
	}
	else if(code == SIGKILL)
	{
		printf("SIGKILL");
	}
	else if(code == SIGQUIT)
	{
		printf("SIGQUIT");
	}
	printf(" signal. See you soon...\n");
	shutdown = TRUE;
}

int main(int argc, char *argv[])
{
	int optchr;
	signal(SIGKILL, handle_quit_signal);
	signal(SIGTERM, handle_quit_signal);
	signal(SIGWINCH, handle_screen_change);
	signal(SIGINT, handle_quit_signal);
	signal(SIGQUIT, handle_quit_signal);
	while((optchr = getopt(argc, argv, "hgnvu:")) != EOF)
	{
		switch(optchr)
		{
			case 'h':
				printf("Help:\n\
-h             Shows the parameter list.\n\
-g             Use GMT time and date.\n\
-u INTERVAL    Change the update interval (ms).\n\
-n             Disables the color mode.\n\
-v             Shows version info and quits.\n");
				return 0;
				break;
			case 'g':
				gmt = TRUE;
				break;
			case 'u':
				update_time = atoi(optarg);
				break;
			case 'n':
				no_color = TRUE;
				break;
			case 'v':
				printf("cclock %s, built on " __DATE__ ".\nAuthor: Dominik Hackl [ dominik@hackl.dhs.org ]\nHomepage: http://www.sourceforge.com/projects/cclock\nALL PARTS OF CCLOCK ARE UNDER THE TERMS OF GPL. SEE DOCUMENTATION FOR MORE INFO.\n", VERSION);
				return 1;
				break;
			default:
				break;
		}
	}
	return start_clock();
}

